/*
 * Document confidentiel - Diffusion interdite 
 */
package com.evasion.dao.api;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implementation Hibernate du GenericDAO.
 * 
 * @param <T>
 *            le type persistent
 * @param <PK>
 *            le type de la clef primaire de l'objet persistent
 */
public abstract class AbstractJPAGenericDAO<T, PK extends Serializable>
        implements GenericDAO<T, PK> {

    /** LOGGER */
    public static final Logger LOGGER = LoggerFactory.getLogger(AbstractJPAGenericDAO.class.getName());
    private EntityManager em;
    /** persistentClass */
    private final Class<T> persistentClass;

    /**
     * Constructeur.
     * @param  namePU
     */
    @SuppressWarnings("unchecked")
    public AbstractJPAGenericDAO() {
        super();
        final ParameterizedType type = (ParameterizedType) this.getClass().getGenericSuperclass();
        this.persistentClass = (Class<T>) type.getActualTypeArguments()[0];
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void clear() {
        LOGGER.debug("Nettoyage (clear) de la session.");
        this.em.clear();
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    @Override
    public List<T> findAll() {
        Query query = this.em.createQuery("SELECT e FROM " + this.persistentClass.getSimpleName() + " e");
        final List<T> result = (List<T>) query.getResultList();
        LOGGER.debug("La recherche de toute les entités de type {} a retourné {} résultats",
                new Object[]{this.persistentClass, result.size()});
        return result;
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public T findById(final PK id) {
        final T result = (T) this.em.find(this.persistentClass,
                id);
        LOGGER.debug("La recherche de l'entite de type {} avec l'id {} a retourn� {}",
                new Object[]{this.persistentClass, id, result});
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public void flush() {
        LOGGER.debug( "Synchronisation (flush) de la session counrante.");
        this.em.flush();
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public T getReference(final PK id) {
        LOGGER.debug(                "R�cup�ration d'une r�f�rence � l'entite de type {} avec l'id {}",
                new Object[]{this.persistentClass, id});
        return (T) this.em.getReference(persistentClass, id);
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public T merge(final T entity) {
        LOGGER.debug("Merging entity {}", new Object[]{entity});
        return (T) em.merge(entity);
    }

    /**
     * {@inheritDoc}
     */
    public void persist(final T entity) {
        LOGGER.debug( "Persisting entity {}", entity);
        this.em.persist(entity);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void remove(T entity) {
        LOGGER.debug( "Suppression de l'entite {}", entity);
        this.em.remove(entity);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public EntityManager getEntityManager() {
        return em;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setEntityManager(EntityManager em) {
        this.em = em;
    }
}
